﻿using Battleship.Drawings;
using System.Text;

namespace Battleship.CollectionGenericObjects;

/// <summary>
/// Класс-хранилище коллекций
/// </summary>
/// <typeparam name="T"></typeparam>
public class StorageCollection<T>
    where T : DrawingWarship
{
    /// <summary>
    /// Словарь (хранилище) с коллекциями 
    /// </summary>
    private Dictionary<string, ICollectionGenericObjects<T>> _storages;

    /// <summary>
    /// Возвращение списка названий коллекции
    /// </summary>
    public List<string> Keys => _storages.Keys.ToList();

    /// <summary>
    /// Конструктор
    /// </summary>
    public StorageCollection()
    {
        _storages = new Dictionary<string, ICollectionGenericObjects<T>>();
    }

    private readonly string _collectionKey = "CollectionsStorage";

    private readonly string _separatorForKeyValue = "|";

    private readonly string _separatorItems = ";";

    /// <summary>
    /// Добавление коллекции в хранилище
    /// </summary>
    /// <param name="name">Название коллекции</param>
    /// <param name="collectionType">Тип коллекции</param>
    public void AddCollection(string name, CollectionType collectionType)
    {
        if (string.IsNullOrEmpty(name) || _storages.ContainsKey(name))
        {
            return;
        }
        // TODO Прописать логику для добавления
        if (collectionType == CollectionType.List)
        {
            _storages.Add(name, new ListGenericObjects<T>());
        }
        if (collectionType == CollectionType.Massive)
        {
            _storages.Add(name, new MassiveGenericObjects<T>());
        }
    }

    /// <summary>
    /// Удаление коллекции
    /// </summary>
    /// <param name="name"></param>
    public void DelCollection(string name)
    {
        // TODO Прописать логику для удаления коллекции
        if (!_storages.ContainsKey(name)) 
            return;
        _storages.Remove(name);
    }

    /// <summary>
    /// Доступ к коллекции 
    /// </summary>
    /// <param name="name"></param>
    /// <returns></returns>
    public ICollectionGenericObjects<T> this[string name]
    {
        get
        {
            // TODO Продумать логику получения объекта
            if (_storages.ContainsKey((string)name))
            {
                return _storages[name];
            }
            return null;
        }
    }

    /// <summary>
    /// Сохранение информации по кораблям в хранилище в файл
    /// </summary>
    /// <param name="filname"></param>
    /// <returns></returns>
    public bool SaveData(string filename)
    {
        if (_storages.Count == 0)
        {
            return false;
        }

        if (File.Exists(filename))
        {
            File.Delete(filename);
        }

        using (StreamWriter writer = new StreamWriter(filename))
        {
            writer.Write(_collectionKey);
            foreach (KeyValuePair<string, ICollectionGenericObjects<T>> value in _storages)
            {
                writer.Write(Environment.NewLine);
                // не сохраняем пустые коллекции
                if (value.Value.Count == 0)
                {
                    continue;
                }

                writer.Write(value.Key);
                writer.Write(_separatorForKeyValue);
                writer.Write(value.Value.GetCollectionType);
                writer.Write(_separatorForKeyValue);
                writer.Write(value.Value.MaxCount);
                writer.Write(_separatorForKeyValue);

                foreach (T? item in value.Value.GetItems())
                {
                    string data = item?.GetDataForSave() ?? string.Empty;
                    if (string.IsNullOrEmpty(data))
                    {
                        continue;
                    }

                    writer.Write(data);
                    writer.Write(_separatorItems);
                }
            }
        }
        return true;
    }

    /// <summary>
    /// Загрузка информации по кораблям в хранилище из файла
    /// </summary>
    /// <param name="filename"></param>
    /// <returns></returns>
    public bool LoadData(string filename)
    {
        if (!File.Exists(filename))
        {
            return false;
        }

        using (StreamReader reader = File.OpenText(filename))
        {
            string str = reader.ReadLine();

            if (str == null || str.Length == 0)
            {
                return false;
            }

            if (!str.StartsWith(_collectionKey))
            {
                //если нет такой записи, то это не те данные
                return false;
            }

            _storages.Clear();
            string strs = "";
            while ((strs = reader.ReadLine()) != null)
            {
                string[] record = strs.Split(_separatorForKeyValue, StringSplitOptions.RemoveEmptyEntries);
                if (record.Length != 4)
                {
                    continue;
                }

                CollectionType collectionType = (CollectionType)Enum.Parse(typeof(CollectionType), record[1]);
                ICollectionGenericObjects<T>? collection = StorageCollection<T>.CreateCollection(collectionType);
                if (collection == null)
                {
                    return false;
                }

                collection.MaxCount = Convert.ToInt32(record[2]);

                string[] set = record[3].Split(_separatorItems, StringSplitOptions.RemoveEmptyEntries);
                foreach (string elem in set)
                {
                    if (elem?.CreateDrawingWarship() is T bulldozer)
                    {
                        if (collection.Insert(bulldozer) == -1)
                        {
                            return false;
                        }
                    }
                }

                _storages.Add(record[0], collection);
            }
        }
        return true;
    }


    /// <summary>
    /// Создание коллекции по типу
    /// </summary>
    /// <param name="collectionType"></param>
    /// <returns></returns>
    private static ICollectionGenericObjects<T>? CreateCollection(CollectionType collectionType)
    {
        return collectionType switch
        {
            CollectionType.Massive => new MassiveGenericObjects<T>(),
            CollectionType.List => new ListGenericObjects<T>(),
            _ => null,
        };
    }
}
